/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LMDBAL_CACHE_H
#define LMDBAL_CACHE_H

#include <map>
#include <set>

#include "storage.h"

namespace LMDBAL {

template <class K, class V>
class Cache : public Storage<K, V> {
    friend class Base;
    enum class Mode {       //it's a cache state when we:
        nothing,            //  - know nothing about records in database on disk
        size,               //  - know just an amount of records
        full                //  - shure that our cache is equal to the database on disk
    };
protected:
    Cache(const std::string& name, Base* parent);
    ~Cache() override;

private:
    void handleMode() const;

public:
    virtual void addRecord(const K& key, const V& value) override;
    virtual bool forceRecord(const K& key, const V& value) override;
    virtual void changeRecord(const K& key, const V& value) override;
    virtual void removeRecord(const K& key) override;
    virtual bool checkRecord(const K& key) const override;
    virtual V getRecord(const K& key) const override;
    virtual uint32_t count() const override;

    using Storage<K, V>::drop;
    virtual int drop(MDB_txn * transaction) override;
    virtual std::map<K, V> readAll() const override;
    virtual void replaceAll(const std::map<K, V>& data) override;
    virtual uint32_t addRecords(const std::map<K, V>& data, bool overwrite = false) override;

protected:
    Mode* mode;
    std::map<K, V>* cache;
    std::set<K>* abscent;
    uint32_t* sizeDifference;
};

}

#include "cache.hpp"

#endif // LMDBAL_CACHE_H
